#!/usr/bin/python3
# -*- coding:utf-8 -*

import os, sys
import re

if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
    application_path = os.path.dirname(sys.executable)
    os.chdir(application_path)
else:
    application_path = os.path.dirname(os.path.abspath(__file__))
    os.chdir(application_path)

prefix = application_path

print("\nProcessing...")

FIN = "annotations.xml"
FLAG = 1 if os.path.exists(FIN) else 0
FOUT = "annotations-mod.xml"
ID_START_TRACK = re.compile('(?<=\<track id\=")\d{1,10}(?=" )')
ID_END_TRACK = re.compile('\<\/track\>')

TRACK_IDX = []
track_ids = []
track_lines = []
track_end_lines = []
bad_start_track_lines = []
bad_counter_list = []
bad_lines_idx = []
bad_idx = 0

try:
	if not FLAG:
		print("\nNo annotation file:", prefix + "/" + FIN, "\n")
		raise Exception("No annotation file")
	else:
		with open(FIN, encoding="utf8") as f:
			n = [line for line in f]
		if len(n) == 0:
			raise Exception("\nNo data in the file: " + prefix + "/" + FIN)
		else:
			TRACK_IDX = [i for i,v in enumerate(n) if re.search("  \<[\/]*track[ ]*.*\>", v)]
			if len(TRACK_IDX)==0:
				print("No tracks detected in annotations.xml. Nothing to do.")
				os._exit(1)
			
			print("")
			for i,l in enumerate(n):
				if i >= TRACK_IDX[0] and i <= TRACK_IDX[-1]:
					if len(ID_START_TRACK.findall(l)) > 0:
						curr_track = ID_START_TRACK.findall(l)[-1]
						track_ids.append(curr_track)
						track_lines.append(i)
					if len(ID_END_TRACK.findall(l)) > 0:
						track_end_lines.append(i)
					if len(track_lines)>0 and i==track_lines[-1]+1 and l.find(' outside="1"')>0:
						bad_start_track_lines.append(i-1)
						bad_counter_list.append(bad_idx)
						bad_idx=bad_idx+1
						print('Line start for "bad" Track ID %s: %s\n' % (str(curr_track),str(i)))	
				
			if len(bad_counter_list)>0:
				for i in bad_counter_list:
					bad_lines_idx.extend(list(range(bad_start_track_lines[i], track_end_lines[track_lines.index(bad_start_track_lines[i])] + 1)))
			
			ids=0
			with open(FOUT, encoding="utf8", mode="w") as h:
				print('\nDeleting "bad" tracks, renumerating "good" tracks and saving annotation file...\n')
				for i,l in enumerate(n):
					if i not in bad_lines_idx:
						if len(ID_START_TRACK.findall(l)) > 0:
							l = re.sub('(?<=\<track id\=")\d{1,10}(?=" )', str(ids), l)
							ids=ids+1
						h.write(l)
				print("Updated annotation xml file saved to:", prefix + "/" + FOUT)
except Exception as e:
	print("Error: " + str(e).strip())
	input("Press ENTER...")
